using NeoTicker;
using System.Collections;
using System.Reflection;
using System;
using KreslikLib.Common.OutputForm;

namespace KreslikLib.Neo.JukeBox
{
    /// <summary>
    /// NTIndicatorJukeBox is used to facilitate multiple application of the same IDL indicator across multiple function windows.
    /// </summary>
    /// <typeparam name="T">Type of the Song class</typeparam>
    /// <typeparam name="U">Type of the Common object class</typeparam>
    public class NTIndicatorJukeBox<T, U>
        where T : Song<U>
        where U : class
    {
        private ArrayList JukeBox;
        private U CommonObj;

        private int UniqueID;
        private int SongIndex;

        public NTIndicatorJukeBox()
        {
            JukeBox = new ArrayList();
            CommonObj = GetNewInstance<U>();
        }

        // find out whether the called Song exists,
        // if it does not, create a new Song
        // and finally call the existing Song
        public double IDLCallSong(NTIndicatorObjects N)
        {
            UniqueID = N.ItSelf().UniqueId;
            SongIndex = GetSongIndexByID(UniqueID);

            if (SongIndex < 0)
            {
                SongIndex = JukeBox.Add(GetNewInstance<T>());
                ((Song<U>)JukeBox[SongIndex]).SetupSong(UniqueID, CommonObj, JukeBox);
            }

            // get the current Song from the JukeBox:
            Song<U> currentSong = (Song<U>)JukeBox[SongIndex];

            return IDLCallExistingSong(N, currentSong);
        }

        private static double IDLCallExistingSong(NTIndicatorObjects N, Song<U> currentSong)
        {
            // assign a return value in case something goes wrong with the Song execution:
            double iDLCallReturn = 0;

            // do not call Songs that have thrown an exception before:
            if (currentSong.SongException == null)
            {
                try
                {
                    if (N.ItSelf().RemoveCall)
                    {
                        currentSong.RemoveCall();
                    }
                    else
                    {
                        iDLCallReturn = currentSong.StandardCall(N);
                    }
                }
                catch (Exception ex)
                {
                    currentSong.SongException = ex;
                    string errorHeader = "Error in ID " + currentSong.UniqueID;
                    OutputWindow.Show(errorHeader, errorHeader + "\n" + "Indicator name: " + N.ItSelf().IndicatorName + "\n" + ex.Message);
                }
            }

            return iDLCallReturn;
        }

        private static R GetNewInstance<R>()
        {
            R returnObject;
            Type returnObjectType = typeof(R);

            returnObject = (R)returnObjectType.InvokeMember
                (returnObjectType.Name,
                BindingFlags.CreateInstance | BindingFlags.Instance | BindingFlags.Public,
                null, null, null);

            return returnObject;
        }

        private int GetSongIndexByID(int ID)
        {
            for (int i = 0; i < JukeBox.Count; i++)
            {
                if (JukeBox[i] != null && ((Song<U>)JukeBox[i]).UniqueID == ID)
                {
                    return i;
                }
            }

            return -1;
        }
    }
}