using System.Collections;
using NeoTicker;
using System;

namespace KreslikLib.Neo.JukeBox
{

    /// <summary>
    /// Song is a class that an IDL indicator must inherit. It represents a unique instance of the IDL indicator.
    /// </summary>
    /// <typeparam name="V">V is a type of class that is shared among all unique indicator instances.</typeparam>
    public abstract class Song<V>
        where V : class
    {
        private bool locked;

        public Song()
        {
            // unlock the class so that we can set it up:
            locked = false;
            songException = null;
        }

        public void SetupSong(int UniqueID, V CommonObj, ArrayList JukeBox)
        {
            // make sure that we don't accidentally assign a value or reference
            // to these vital objects later:
            if (!locked)
            {
                uniqueID = UniqueID;
                commonObj = CommonObj;
                jukeBox = JukeBox;
                locked = true;
            }
            else
            {
                throw new Exception("The " + this.GetType().Name + " object is locked.");
            }
        }

        private Exception songException;

        public Exception SongException
        {
            get { return songException; }
            set { songException = value; }
        }

        private int uniqueID;

        /// <summary>
        /// Read only property UniqueID is set upon the Song's instantiation.
        /// </summary>
        public int UniqueID
        {
            get { return uniqueID; }
        }

        private V commonObj;

        /// <summary>
        /// Read only property CommonObj is set upon the Song's instantiation.
        /// </summary>
        protected V CommonObj
        {
            get { return commonObj; }
        }

        private ArrayList jukeBox;

        /// <summary>
        /// Read only property Songs is set upon the Song's instantiation.
        /// </summary>
        protected ArrayList JukeBox
        {
            get { return jukeBox; }
        }

        public abstract double StandardCall(NTIndicatorObjects N);
        public abstract void RemoveCall();
    } 
}